﻿using HeuristicLab.Algorithms.PushGP.Interpreter;
using HeuristicLab.Algorithms.PushGP.Stack;
using HeuristicLab.Common;

namespace HeuristicLab.Algorithms.PushGP.Expressions
{
    public abstract class DoTimesExpression : LoopExpression
    {
        public DoTimesExpression()
        { }

        public DoTimesExpression(LoopExpression origin, Cloner cloner) : base(origin, cloner)
        { }

        protected override bool HasInsufficientArguments(IInterpreter interpreter, IStack<Expression> sourceStack)
        {
            return interpreter.IntegerStack.Count < 1 ||
                   sourceStack.Count == 0 ||
                   interpreter.IntegerStack.Top <= 0;
        }

        protected override void InitState(IInterpreter interpreter, IStack<Expression> sourceStack)
        {
            body = sourceStack.Pop();
            destinationIndex = interpreter.IntegerStack.Pop();
            currentIndex = 1;
            incrementor = 1;
        }

        protected override void PushIteration(IInterpreter interpreter)
        {
            interpreter.ExecStack.Push(this, body);
        }

        protected override void PushLastIteration(IInterpreter interpreter)
        {
            interpreter.ExecStack.Push(body);
        }
    }

    /// <summary>
    /// Like CODE.DO*COUNT but does not push the loop counter.
    /// This should be implemented as a macro that expands into CODE.DO*RANGE, similarly to the implementation
    /// of CODE.DO*COUNT, except that a call to INTEGER.POP should be tacked on to the front of the loop body code
    /// in the call to CODE.DO*RANGE.
    /// </summary>

    public class CodeDoTimesExpression : DoTimesExpression
    {
        public CodeDoTimesExpression()
        { }

        protected CodeDoTimesExpression(CodeDoTimesExpression origin, Cloner cloner) : base(origin, cloner)
        { }

        public override IDeepCloneable Clone(Cloner cloner)
        {
            return new CodeDoTimesExpression(this, cloner);
        }

        protected override string InitStringRepresentation()
        {
            return "CODE.DO*TIMES";
        }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter, interpreter.CodeStack);
        }
    }

    /// <summary>
    /// Like EXEC.DO*COUNT but does not push the loop counter. This should be implemented as a macro that expands into EXEC.DO*RANGE,
    /// similarly to the implementation of EXEC.DO*COUNT, except that a call to INTEGER.POP should be tacked on to the front of the
    /// loop body code in the call to EXEC.DO*RANGE. This call to INTEGER.POP will remove the loop counter, which will have been
    /// pushed by EXEC.DO*RANGE, prior to the execution of the loop body.
    /// </summary>

    public class ExecDoTimesExpression : DoTimesExpression
    {
        public ExecDoTimesExpression()
        { }

        protected ExecDoTimesExpression(ExecDoTimesExpression origin, Cloner cloner) : base(origin, cloner)
        { }

        public override IDeepCloneable Clone(Cloner cloner)
        {
            return new ExecDoTimesExpression(this, cloner);
        }

        protected override string InitStringRepresentation()
        {
            return "EXEC.DO*TIMES";
        }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter, interpreter.ExecStack);
        }
    }
}
