﻿using System;
using HeuristicLab.Algorithms.PushGP.Generators;
using HeuristicLab.Algorithms.PushGP.Interpreter;
using HeuristicLab.Algorithms.PushGP.Stack;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Random;

namespace HeuristicLab.Algorithms.PushGP.Expressions
{
    public abstract class ERCExpression<T> : Expression
    {
        private readonly T value;

        public ERCExpression(T value)
        {
            this.value = value;
        }

        protected void Eval(IStack<T> stack)
        {
            stack.Push(value);
        }

        /// <summary>
        /// Does not make sense
        /// </summary>
        /// <returns></returns>
        protected override string InitStringRepresentation()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Does not make sense
        /// </summary>
        /// <returns></returns>
        protected override int InitId()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Does not make sense
        /// </summary>
        /// <returns></returns>
        public override IDeepCloneable Clone(Cloner cloner)
        {
            throw new NotImplementedException();
        }
    }

    public class IntegerERCExpression : ERCExpression<long>
    {
        public IntegerERCExpression() : base(IntegerGenerator.RandomInteger())
        {
        }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.IntegerStack);
        }
    }

    public class FloatERCExpression : ERCExpression<double>
    {
        protected bool isInitialized = false;

        public FloatERCExpression() : base(FloatGenerator.RandomFloat())
        {
        }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.FloatStack);
        }
    }

    public class BooleanERCExpression : ERCExpression<bool>
    {
        public BooleanERCExpression() : base(BooleanGenerator.RandomBoolean())
        {
        }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.BooleanStack);
        }
    }

    public class NameERCExpression : ERCExpression<string>
    {
        private static IRandom random = new FastRandom();

        public NameERCExpression() : base(NameGenerator.RandomName())
        {
        }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.NameStack);
        }
    }
}
